# Copyright (C) 2010-04-10 Adam Sloboda
# If you want to use the code, just let me know.

require 'gtk2'

require 'rubygems'
require 'midiator'
$midi = MIDIator::Interface.new
$midi.autodetect_driver

class MidiList
  attr_reader :view

  ID = 0
  LINE = 1

  def initialize
    @model = Gtk::ListStore.new(String, String)
    @view = Gtk::TreeView.new(@model)

    renderer = Gtk::CellRendererText.new
    column = Gtk::TreeViewColumn.new('id', renderer, :text => ID)
    @view.append_column(column)
    column = Gtk::TreeViewColumn.new('device', renderer, :text => LINE)
    @view.append_column(column)
    update
  end

  def refresh(list)
    @model.clear
    list.each do |i|
      puts i
      iter = @model.append
      iter[ID] = i.match(/[0-9]+/).to_s
      iter[LINE] = i.strip
    end
  end

  def current
    @view.selection.selected[ID]
  end
end

class InputList < MidiList
  def update
    refresh `aconnect -i | grep '^client'`
  end
end

class OutputList < MidiList
  def update
    refresh `aconnect -o | grep '^client'`
  end
end

class MainWindow
  def initialize
    @in = InputList.new
    @out = OutputList.new

    # buttons: refresh, connect
    h = Gtk::HBox.new
    b = Gtk::Button.new('Refresh')
    b.signal_connect('clicked') { [@in, @out].each { |i| i.update } }
    h.pack_start(b)

    b = Gtk::Button.new('Connect')
    b.signal_connect('clicked') do
      begin
        cmd = "aconnect #{@in.current} #{@out.current}"
        puts "running \"#{cmd}\""
        system(cmd)
        Thread.new { $midi.play(84, 0.5) }
      rescue:
      end
    end
    h.pack_start(b)

    # midi selection
    midi = Gtk::VBox.new
    v = Gtk::VBox.new
    v.pack_start(@in.view)
    v.pack_start(@out.view)
    midi.pack_start(v)
    midi.pack_start(h)

    # tone grid
    g = Grid.new
    midi.pack_start(g)
    h = Gtk::HBox.new
    b = Gtk::Button.new('Clear')
    b.signal_connect('clicked') { g.clear }
    h.pack_start(b)
    b = Gtk::ToggleButton.new('Play')
    b.signal_connect('toggled') { g.toggle }
    h.pack_start(b)
    midi.pack_start(h)

    # window
    w = Gtk::Window.new
    w.signal_connect('destroy') { Gtk.main_quit }
    w.add(midi)
    w.show_all
  end
end

class Grid < Gtk::VBox
  TIMING = 0.2

  def initialize
    @play = false
    @tones = Array.new(16) { Array.new(16) }
    @status = Array.new(16)

    @t = Gtk::Table.new(16, 16, true)
    @ind = Gtk::Table.new(1, 16, true)

    16.times do |x|
      16.times do |y|
        b = Gtk::ToggleButton.new(y.to_s)
        @tones[x][y] = b
        @t.attach_defaults(b, x, x+1, y, y+1)

        b.signal_connect('toggled') do #|w|
          Thread.new { $midi.play(50+(16-b.label.to_i), 0.1) }
        end
      end

      # play indicator
      b = Gtk::ToggleButton.new('^')
      @status[x] = b
      @ind.attach_defaults(b, x, x+1, 0, 1)
    end

    super
    pack_start(@t)
    pack_start(@ind)
  end

  def play
    while @play
      @tones.each_index do |i|
        @status[i-1].active = false if i > 0
        @status[i].active = true
        @tones[i].each do |b|
          Thread.new { $midi.play(50+(16-b.label.to_i), TIMING) } if b.active?
        end

        while Gtk.events_pending?
          Gtk.main_iteration
        end
        sleep TIMING
      end

      @status[-1].active = false
    end
  end

  def clear
    @tones.each { |arr| arr.each { |b| b.active = false } }
  end

  def toggle
    @play = !@play
    play if @play
  end
end

MainWindow.new
Gtk.main

#ln -s libasound.so.2 libasound.so
